function [e,phi,Hmatrix]=schrod(length,n,v,num_sol)
% [energy, phi_fun, Hmatrix]=schrod(length,number,potential,sol_num)
% solves time-independent Schrodinger equation for energy eigenvalues
% and eigenfunctions for an electron in arbitrary potential region
% bounded by 0<=x<=length. Potential is infinite outside this region.
% Note: phi columns are of size n+1, whereas Hmatrix is of size (n-1)x(n-1)
%       because phi vanishes at both ends.
% 
% length	    length of region (nm)
% n				number of sample points 
% v				potential inside region (eV)
% sol_num		number of solutions sought
%
% e				energy eigenvalue (eV)
% phi			eigenfunction with eigenvalue = e
%

hbar=1.054571596;              %Planck's constant (x10^34 J s)
hbar2=hbar^2;
echarge=1.602176462;		   %electron charge (x10^19 C)
emass=9.10938188;              %electron mass (x10^31 kg)

const=hbar2/emass/echarge;     %(hbar^2)/(echarge*1nm^2*m0)
deltax=length/n;			   %x-increment = length(nm)/n
deltax2=deltax^2;
const=const/deltax2;

d(1:n-1)=v(2:n)+const;         %diagonal matrix element
offd(1:n-1)=const/2;           %off-diagonal matrix element

t1=-offd(2:n-1);
t2=d(1:n-1);

Hmatrix=zeros(n-1,n-1);         %create (zero) Hamiltonian matrix
Hmatrix=Hmatrix+diag(t1,1);     %upper diagonal of Hamiltonian matrix
Hmatrix=Hmatrix+diag(t1,-1);	%add lower diagonal of Hamiltonian matrix
Hmatrix=Hmatrix+diag(t2,0);		%add diagonal of Hamiltonian matrix

[phi,te]=eigs(Hmatrix,num_sol,'sm');		%use matlab function eigs to
                                            %find num_sol eigenfunctions
                                            %and eigenvalues
e = diag(te);                   %extract diagonal from matrix
phi=[zeros(1,num_sol);phi;zeros(1,num_sol)]; %wave function is zero
                                             %at x = 0 and x = length
return